using System;
using System.Runtime.InteropServices;
using System.ComponentModel;
using Outlook = Microsoft.Office.Interop.Outlook;
using System.Windows.Forms;
using System.Threading;
using System.IO;

namespace SynchronizationTemplate
{
    /// <summary>
    ///   Add-in Express Add-in Module
    /// </summary>
    [GuidAttribute("CE5031A3-1EDC-4D51-8AA5-BF0238A7CD1B"), ProgId("SynchronizationTemplate.AddinModule"), RunInstaller(true)]
    public class AddinModule : AddinExpress.MSO.ADXAddinModule
    {
		[DllImport("user32.dll")]
		public static extern IntPtr PostMessage(IntPtr hwnd, uint wMsg, Int32 wParam, Int32 lParam);

		public AddinModule()
        {
            InitializeComponent();
        }
 
        #region Component Designer generated code
        /// <summary>
        /// Required by designer
        /// </summary>
        private System.ComponentModel.IContainer components;
 
        /// <summary>
        /// Required by designer support - do not modify
        /// the following method
        /// </summary>
        private void InitializeComponent()
        {
			// 
			// AddinModule
			// 
			this.AddinName = "SynchronizationTemplate";
			this.SupportedApps = AddinExpress.MSO.ADXOfficeHostApp.ohaOutlook;
			this.AddinInitialize += new AddinExpress.MSO.ADXEvents_EventHandler(this.AddinModule_AddinInitialize);
			this.AddinBeginShutdown += new AddinExpress.MSO.ADXEvents_EventHandler(this.AddinModule_AddinBeginShutdown);
			this.AddinStartupComplete += new AddinExpress.MSO.ADXEvents_EventHandler(this.AddinModule_AddinStartupComplete);

		}
        #endregion
 
        #region ADX automatic code
 
        // Required by Add-in Express - do not modify
        // the methods within this region
 
        public override System.ComponentModel.IContainer GetContainer()
        {
            if (components == null)
                components = new System.ComponentModel.Container();
            return components;
        }
 
        [ComRegisterFunctionAttribute]
        public static void AddinRegister(Type t)
        {
            AddinExpress.MSO.ADXAddinModule.ADXRegister(t);
        }
 
        [ComUnregisterFunctionAttribute]
        public static void AddinUnregister(Type t)
        {
            AddinExpress.MSO.ADXAddinModule.ADXUnregister(t);
        }
 
        public override void UninstallControls()
        {
            base.UninstallControls();
        }

        #endregion

		private Outlook.MAPIFolder contacts = null;
		private static AddinModule module;
		private static bool requestAbort = false;
		private static bool exitProcess = false;
		private Thread myThread;

		private void AddinModule_AddinStartupComplete(object sender, System.EventArgs e)
		{
			Outlook._NameSpace ns = null;
			try
			{
				ns = OutlookApp.GetNamespace("MAPI");
				contacts = ns.GetDefaultFolder(Outlook.OlDefaultFolders.olFolderContacts);

				myThread = new Thread(new ThreadStart(ThreadProc));
				myThread.Start();
			}
			finally
			{
				if (ns != null)
					Marshal.ReleaseComObject(ns);
			}
		}

		private void AddinModule_AddinBeginShutdown(object sender, System.EventArgs e)
		{
			ProgressDialog pd = new ProgressDialog();
			try
			{
				requestAbort = true;
				if (myThread.ThreadState == ThreadState.Running)
				{
					myThread.Suspend();
					if (MessageBox.Show("Outlook is currently sinchronizing contacts. Would you like to close Outlook anyway?", 
						this.AddinName, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
					{
						pd.Show(); pd.Update();
						myThread.Resume();
					}
					else
					{
						exitProcess = true;
						myThread.Resume();
						myThread.Abort();
					}
					myThread.Join();
				}
				else if (myThread.ThreadState == ThreadState.WaitSleepJoin)
				{
					myThread.Abort();
					myThread.Join();
				}
			}
			finally
			{
				if (pd != null)
					pd.Dispose();
				if (contacts != null)
					Marshal.ReleaseComObject(contacts);
			}
		}

        public Outlook._Application OutlookApp
        {
            get
            {
                return (HostApplication as Outlook._Application);
            }
        }

		private static void ThreadProc() 
		{
			int i;
			while(!requestAbort)
			{
				Thread.Sleep(5000);
				if (!requestAbort)
				{
					i = 1;
					//FileStream fsDst = null;
					//StreamWriter swDst = null;
					try
					{
						object item;
						
						/*if (File.Exists("D:\\log.txt"))
							fsDst = new FileStream("D:\\log.txt", FileMode.Truncate, FileAccess.Write);
						else
							fsDst = new FileStream("D:\\log.txt", FileMode.CreateNew, FileAccess.Write);
						swDst = new StreamWriter(fsDst);*/

						while((i <= module.contacts.Items.Count) && (exitProcess == false))
						{
							item = null;
							try
							{
								item = module.contacts.Items[i];
								// here do something with an Outlook item
								if (item is Outlook.ContactItem)
								{
									//swDst.WriteLine((item as Outlook.ContactItem).LastName + ", " +
									//	(item as Outlook.ContactItem).FirstName);
								}
							}
							catch(Exception e)
							{
								//swDst.WriteLine("Error: " + e.Message);
							}
							finally
							{
								i++;
								if (item != null)
									Marshal.ReleaseComObject(item);
							}
						}
					}
					catch(ThreadAbortException e)
					{
						throw e;
					}
					catch
					{
						// nothing
					}
					finally
					{
						/*if (swDst != null) 
							swDst.Close();
						if (fsDst != null) 
							fsDst.Close();*/
					}
				}
			}
		}

		private void AddinModule_AddinInitialize(object sender, System.EventArgs e)
		{
			module = this;
		}
	
	}
}

